package javax.swing.text;

import java.awt.Graphics;
import java.awt.Shape;
import javax.swing.event.DocumentEvent;
import java.util.Vector;

public class ZoneView extends BoxView
{
    int maxZoneSize;
    int maxZonesLoaded;
    Vector<View> loadedZones;
    
    public ZoneView(Element element, int n) {
        super(element, n);
        this.maxZoneSize = 8192;
        this.maxZonesLoaded = 3;
        this.loadedZones = new Vector();
    }
    
    public int getMaximumZoneSize() {
        return this.maxZoneSize;
    }
    
    public void setMaximumZoneSize(int maxZoneSize) {
        this.maxZoneSize = maxZoneSize;
    }
    
    public int getMaxZonesLoaded() {
        return this.maxZonesLoaded;
    }
    
    public void setMaxZonesLoaded(int maxZonesLoaded) {
        if (maxZonesLoaded < 1) {
            throw new IllegalArgumentException("ZoneView.setMaxZonesLoaded must be greater than 0.");
        }
        this.maxZonesLoaded = maxZonesLoaded;
        this.unloadOldZones();
        return;
    }
    
    protected void zoneWasLoaded(View obj) {
        this.loadedZones.addElement(obj);
        this.unloadOldZones();
    }
    
    void unloadOldZones() {
        while (this.loadedZones.size() > this.getMaxZonesLoaded()) {
            View view = (View)this.loadedZones.elementAt(0);
            this.loadedZones.removeElementAt(0);
            this.unloadZone(view);
        }
    }
    
    protected void unloadZone(View view) {
        view.removeAll();
    }
    
    protected boolean isZoneLoaded(View view) {
        return view.getViewCount() > 0;
    }
    
    protected View createZone(int n, int n2) {
        Document document = this.getDocument();
        ZoneView.Zone zone;
        try {
            zone = new ZoneView.Zone(this.getElement(), document.createPosition(n), document.createPosition(n2));
        }
        catch (BadLocationException ex) {
            throw new StateInvariantError(ex.getMessage());
        }
        return zone;
    }
    
    @Override
    protected void loadChildren(ViewFactory viewFactory) {
        this.getDocument();
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        this.append(this.createZone(startOffset, endOffset));
        this.handleInsert(startOffset, endOffset - startOffset);
    }
    
    @Override
    protected int getViewIndexAtPosition(int n) {
        int viewCount = this.getViewCount();
        if (n == this.getEndOffset()) {
            return viewCount - 1;
        }
        int i = 0;
        while (i < viewCount) {
            View view = this.getView(i);
            if (n >= view.getStartOffset() && n < view.getEndOffset()) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    void handleInsert(int n, int n2) {
        int viewIndex = this.getViewIndex(n, Position.Bias.Forward);
        View view = this.getView(viewIndex);
        int startOffset = view.getStartOffset();
        int endOffset = view.getEndOffset();
        if (endOffset - startOffset > this.maxZoneSize) {
            this.splitZone(viewIndex, startOffset, endOffset);
        }
    }
    
    void handleRemove(int n, int n2) {
    }
    
    void splitZone(int n, int i, int b) {
        this.getElement().getDocument();
        Vector<View> vector = new Vector();
        do {
            i = Math.min(this.getDesiredZoneEnd(i), b);
            vector.addElement(this.createZone(i, i));
        } while (i < b);
        this.getView(n);
        View[] anArray = new View[vector.size()];
        vector.copyInto(anArray);
        this.replace(n, 1, anArray);
    }
    
    int getDesiredZoneEnd(int n) {
        Element element = this.getElement();
        Element element2 = element.getElement(element.getElementIndex(n + this.maxZoneSize / 2));
        int startOffset = element2.getStartOffset();
        int endOffset = element2.getEndOffset();
        if (endOffset - n > this.maxZoneSize && startOffset > n) {
            return startOffset;
        }
        return endOffset;
    }
    
    @Override
    protected boolean updateChildren(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, ViewFactory viewFactory) {
        return false;
    }
    
    @Override
    public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.handleInsert(documentEvent.getOffset(), documentEvent.getLength());
        super.insertUpdate(documentEvent, shape, viewFactory);
    }
    
    @Override
    public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.handleRemove(documentEvent.getOffset(), documentEvent.getLength());
        super.removeUpdate(documentEvent, shape, viewFactory);
    }
    
    class Zone extends AsyncBoxView
    {
        private Position start;
        private Position end;
        
        public Zone(Element element, Position start, Position end) {
            super(element, ZoneView.this.getAxis());
            this.start = start;
            this.end = end;
        }
        
        public void load() {
            if (!this.isLoaded()) {
                this.setEstimatedMajorSpan(true);
                Element element = this.getElement();
                ViewFactory viewFactory = this.getViewFactory();
                int i = element.getElementIndex(this.getStartOffset());
                int elementIndex = element.getElementIndex(this.getEndOffset());
                View[] array = new View[elementIndex - i + 1];
                for (; i <= elementIndex; ++i) {
                    array[i - i] = viewFactory.create(element.getElement(i));
                }
                this.replace(0, 0, array);
                ZoneView.this.zoneWasLoaded(this);
            }
        }
        
        public void unload() {
            this.setEstimatedMajorSpan(true);
            this.removeAll();
        }
        
        public boolean isLoaded() {
            return this.getViewCount() != 0;
        }
        
        @Override
        protected void loadChildren(ViewFactory viewFactory) {
            this.setEstimatedMajorSpan(true);
            Element element = this.getElement();
            int elementIndex = element.getElementIndex(this.getStartOffset());
            int n = element.getElementIndex(this.getEndOffset()) - elementIndex;
            View create = viewFactory.create(element.getElement(elementIndex));
            create.setParent(this);
            float preferredSpan = create.getPreferredSpan(0);
            float preferredSpan2 = create.getPreferredSpan(1);
            if (this.getMajorAxis() == 0) {
                preferredSpan *= (float)n;
            }
            else {
                preferredSpan2 += (float)n;
            }
            this.setSize(preferredSpan, preferredSpan2);
        }
        
        @Override
        protected void flushRequirementChanges() {
            if (this.isLoaded()) {
                super.flushRequirementChanges();
            }
        }
        
        @Override
        public int getViewIndex(int n, Position.Bias bias) {
            n = ((bias == Position.Bias.Backward) ? Math.max(0, n - 1) : n);
            Element element = this.getElement();
            return element.getElementIndex(n) - element.getElementIndex(this.getStartOffset());
        }
        
        @Override
        protected boolean updateChildren(DocumentEvent.ElementChange elementChange, DocumentEvent documentEvent, ViewFactory viewFactory) {
            Element[] childrenRemoved = elementChange.getChildrenRemoved();
            Element[] childrenAdded = elementChange.getChildrenAdded();
            Element element = this.getElement();
            int elementIndex = element.getElementIndex(this.getStartOffset());
            int elementIndex2 = element.getElementIndex(this.getEndOffset() - 1);
            int index = elementChange.getIndex();
            if (index >= elementIndex && index <= elementIndex2) {
                int n = index - elementIndex;
                int min = Math.min(elementIndex2 - elementIndex + 1, childrenAdded.length);
                int min2 = Math.min(elementIndex2 - elementIndex + 1, childrenRemoved.length);
                View[] array = new View[min];
                for (int i = 0; i < min; ++i) {
                    array[i] = viewFactory.create(childrenAdded[i]);
                }
                this.replace(n, min2, array);
            }
            return true;
        }
        
        @Override
        public AttributeSet getAttributes() {
            return ZoneView.this.getAttributes();
        }
        
        @Override
        public void paint(Graphics graphics, Shape shape) {
            this.load();
            super.paint(graphics, shape);
        }
        
        @Override
        public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
            this.load();
            return super.viewToModel(n, n2, shape, array);
        }
        
        @Override
        public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
            this.load();
            return super.modelToView(n, shape, bias);
        }
        
        @Override
        public int getStartOffset() {
            return this.start.getOffset();
        }
        
        @Override
        public int getEndOffset() {
            return this.end.getOffset();
        }
        
        @Override
        public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
            if (this.isLoaded()) {
                super.insertUpdate(documentEvent, shape, viewFactory);
            }
        }
        
        @Override
        public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
            if (this.isLoaded()) {
                super.removeUpdate(documentEvent, shape, viewFactory);
            }
        }
        
        @Override
        public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
            if (this.isLoaded()) {
                super.changedUpdate(documentEvent, shape, viewFactory);
            }
        }
    }
}
